@@ -17,7 +17,7 @@
     use constant _MATH_MPC_T    => 10;
 
 use subs qw( __GNU_MP_VERSION __GNU_MP_VERSION_MINOR __GNU_MP_VERSION_PATCHLEVEL
-             __GMP_CC __GMP_CFLAGS);
+             __GNU_MP_RELEASE __GMP_CC __GMP_CFLAGS);
 
 use overload
     '++'   => \&overload_inc,
@@ -48,7 +48,7 @@ use overload
 
     @Math::GMPf::EXPORT_OK = qw(
 __GNU_MP_VERSION __GNU_MP_VERSION_MINOR __GNU_MP_VERSION_PATCHLEVEL
-__GMP_CC __GMP_CFLAGS
+__GNU_MP_RELEASE __GMP_CC __GMP_CFLAGS
 Rmpf_abs Rmpf_add Rmpf_add_ui Rmpf_ceil Rmpf_clear Rmpf_clear_mpf Rmpf_clear_ptr
 Rmpf_cmp Rmpf_cmp_d Rmpf_cmp_si Rmpf_cmp_ui 
 Rmpf_deref2 Rmpf_div Rmpf_div_2exp Rmpf_div_ui
@@ -67,7 +67,7 @@ TRmpf_out_str
 Rmpf_pow_ui Rmpf_printf 
 Rmpf_random2 Rmpf_reldiff Rmpf_set Rmpf_set_d Rmpf_set_default_prec Rmpf_set_prec 
 Rmpf_set_prec_raw Rmpf_set_q Rmpf_set_si Rmpf_set_str Rmpf_set_ui Rmpf_set_z 
-Rmpf_sgn Rmpf_sprintf Rmpf_sprintf_ret  Rmpf_snprintf Rmpf_snprintf_ret
+Rmpf_sgn Rmpf_sprintf Rmpf_snprintf
 Rmpf_sqrt Rmpf_sqrt_ui Rmpf_sub Rmpf_sub_ui Rmpf_swap Rmpf_trunc 
 Rmpf_ui_div Rmpf_ui_sub Rmpf_urandomb
 fgmp_randseed fgmp_randseed_ui fgmp_randclear
@@ -76,7 +76,7 @@ fgmp_randinit_set fgmp_randinit_default_nobless fgmp_randinit_mt_nobless
 fgmp_randinit_lc_2exp_nobless fgmp_randinit_lc_2exp_size_nobless fgmp_randinit_set_nobless
 fgmp_urandomb_ui fgmp_urandomm_ui
     );
-    our $VERSION = '0.36';
+    our $VERSION = '0.37';
     $VERSION = eval $VERSION;
 
     DynaLoader::bootstrap Math::GMPf $VERSION;
@@ -100,7 +100,7 @@ TRmpf_out_str
 Rmpf_pow_ui Rmpf_printf 
 Rmpf_random2 Rmpf_reldiff Rmpf_set Rmpf_set_d Rmpf_set_default_prec Rmpf_set_prec 
 Rmpf_set_prec_raw Rmpf_set_q Rmpf_set_si Rmpf_set_str Rmpf_set_ui Rmpf_set_z 
-Rmpf_sgn Rmpf_sprintf Rmpf_sprintf_ret Rmpf_snprintf Rmpf_snprintf_ret
+Rmpf_sgn Rmpf_sprintf Rmpf_snprintf
 Rmpf_sqrt Rmpf_sqrt_ui Rmpf_sub Rmpf_sub_ui Rmpf_swap Rmpf_trunc 
 Rmpf_ui_div Rmpf_ui_sub Rmpf_urandomb
 fgmp_randseed fgmp_randseed_ui fgmp_randclear
@@ -273,38 +273,37 @@ sub Rmpf_fprintf {
 }
 
 sub Rmpf_sprintf {
-    push @_, 0 if @_ == 2; # add a dummy third argument
-    die "Rmpf_sprintf must pass 3 arguments: buffer, format string, and variable" if @_ != 3;
-    my $len = wrap_gmp_sprintf(@_);
-    $_[0] = substr($_[0], 0, $len);
-    return $len;
-}
+    my $len;
 
-sub Rmpf_sprintf_ret {
-    push @_, 0 if @_ == 2; # add a dummy third argument
-    die "Rmpf_sprintf_ret must pass 3 arguments: buffer, format string, and variable" if @_ != 3;
-    my $len = wrap_gmp_sprintf(@_);
-    return substr($_[0], 0, $len);
+    if(@_ == 3) {      # optional arg wasn't provided
+      $len = wrap_gmp_sprintf($_[0], $_[1], 0, $_[2]);  # Set missing arg to 0
+    }
+    else {
+      die "Rmpf_sprintf must pass 4 arguments: buffer, format string, variable, buflen" if @_ != 4;
+      $len = wrap_gmp_sprintf(@_);
+    }
+
+    return $len;
 }
 
 sub Rmpf_snprintf {
-    push @_, 0 if @_ == 3; # add a dummy third argument
-    die "Rmpf_snprintf must pass 4 arguments: buffer, bytes written, format string, and variable" if @_ != 4;
-    my $len = wrap_gmp_snprintf(@_);
-    $_[0] = substr($_[0], 0, $_[1] - 1);
-    return $len;
-}
+    my $len;
 
-sub Rmpf_snprintf_ret {
-    push @_, 0 if @_ == 3; # add a dummy third argument
-    die "Rmpf_snprintf_ret must pass 4 arguments: buffer, bytes written, format string, and variable" if @_ != 4;
-    my $len = wrap_gmp_snprintf(@_);
-    return substr($_[0], 0, $_[1] - 1);
+    if(@_ == 4) {      # optional arg wasn't provided
+      $len = wrap_gmp_snprintf($_[0], $_[1], $_[2], 0, $_[3]);  # Set missing arg to 0
+    }
+    else {
+      die "Rmpf_snprintf must pass 5 arguments: buffer, bytes written, format string, variable and buflen" if @_ != 5;
+      $len = wrap_gmp_snprintf(@_);
+    }
+
+    return $len;
 }
 
 sub __GNU_MP_VERSION {return ___GNU_MP_VERSION()}
 sub __GNU_MP_VERSION_MINOR {return ___GNU_MP_VERSION_MINOR()}
 sub __GNU_MP_VERSION_PATCHLEVEL {return ___GNU_MP_VERSION_PATCHLEVEL()}
+sub __GNU_MP_RELEASE {return ___GNU_MP_RELEASE()}
 sub __GMP_CC {return ___GMP_CC()}
 sub __GMP_CFLAGS {return ___GMP_CFLAGS()}
 
@@ -1050,30 +1049,21 @@ __END__
     Returns the number of characters written, or -1 if an error
     occurred.
 
-   $si = Rmpf_sprintf($buffer, $format_string, $var);
+   $si = Rmpf_sprintf($buffer, $format_string, $var, $buflen);
 
     This function (unlike the GMP counterpart) is limited to taking
-    3 arguments - the buffer, the format string, and the variable
-    to be formatted. If there is no variable to be formatted, then the
-    final arg can be omitted - a suitable dummy arg will be passed to
-    the XS code for you. ie the following will work:
-     Rmpf_sprintf($buffer, "hello world\n");
-    $buffer must be large enough to accommodate the formatted string,
-    and is truncated to the length of that formatted string.
-    If you prefer to have the resultant string returned (rather
-    than stored in $buffer), use Rmpf_sprintf_ret instead - which will
-    also leave the length of $buffer unaltered.
+    4 arguments - the buffer, the format string,  the variable to be
+    formatted and the size of the buffer. If there is no variable to
+    be formatted, then the third arg can be omitted - a suitable
+    dummy arg will be passed to the XS code for you. ie the following
+    will work:
+     Rmpf_sprintf($buffer, "hello world", 12);
+    $buffer must be large enough to accommodate the formatted string.
+    The formatted string is placed in $buffer.
     Returns the number of characters written, or -1 if an error
     occurred.
 
-   $string = Rmpf_sprintf_ret($buffer, $format_string, $var);
-
-    As for Rmpf_sprintf, but returns the formatted string, as well as
-    storing it in $buffer. $buffer needs to be large enough to 
-    accommodate the formatted string. The length of $buffer will be
-    unaltered.
-
-   $si = Rmpf_snprintf($buffer, $bytes, $format_string, $var);
+   $si = Rmpf_snprintf($buffer, $bytes, $format_string, $var, $buflen);
 
     Form a null-terminated string in $buffer. No more than $bytes 
     bytes will be written. To get the full output, $bytes must be
@@ -1085,23 +1075,13 @@ __END__
     If $si >= $bytes then the actual output has been truncated to
     the first $bytes-1 characters, and a null appended.
     This function (unlike the GMP counterpart) is limited to taking
-    4 arguments - the buffer, the maximum number of bytes to be
-    returned, the format string, and the variable to be formatted.
-    If there is no variable to be formatted, then the final arg can
+    5 arguments - the buffer, the maximum number of bytes to be
+    returned, the format string, the variable to be formatted and
+    the size of the buffer.
+    If there is no variable to be formatted, then the 4th arg can
     be omitted - a suitable dummy arg will be passed to the XS code
     for you. ie the following will work:
-     Rmpf_snprintf($buffer, 12, "hello world");
-    If you prefer to have the resultant string returned (rather
-    than stored in $buffer), use Rmpf_snprintf_ret instead - which will
-    also leave the length of $buffer unaltered.
-
-   $string = Rmpf_snprintf_ret($buffer, $bytes, $format_string, $var);
-
-    As for Rmpf_snprintf, but returns the formatted string, as well as
-    storing it in $buffer. $buffer needs to be large enough to 
-    accommodate the formatted string. The length of $buffer will be
-    unaltered. The length of $string (as reported by perl's length
-    function) will be no greater than $bytes.
+     Rmpf_snprintf($buffer, 6, "hello world", 12);
 
    ###############################
    ###############################
